'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (component) {
  component.prototype.getDefaultRules = getDefaultRules;
  return component;
};

var _sprintfJs = require('sprintf-js');

var _util = require('../utils/util');

/**
 * 长度验证rules
 * @param  {number} min 最小长度
 * @param  {number} max 最大长度
 * @param  {object} locale 多语言对象
 * @return {array}  rules
 */
// import React from 'react';
function getLenthRule(min, max, locale) {
  var message = void 0;
  return [{
    validator: function validator(rule, value, callback) {
      var errors = [];
      if (value) {
        var valueLength = (0, _util.strlen)(value);
        var flag = true;
        if (min && max) {
          if (valueLength < min || valueLength > max) {
            flag = false;
            message = (0, _sprintfJs.sprintf)(locale.afcCommon.charactersBetwteen, min, max);
          }
        } else if (min) {
          if (valueLength < min) {
            flag = false;
            message = (0, _sprintfJs.sprintf)(locale.afcCommon.charactersMin, min);
          }
        } else if (max) {
          if (valueLength > max) {
            flag = false;
            message = (0, _sprintfJs.sprintf)(locale.afcCommon.charactersMax, max);
          }
        }
        if (!flag) {
          errors.push({
            message: message
          });
        }
      }
      callback(errors);
    }
  }];
}
/**
 * 获取默认的rules，如required等
 * @param  {string} componentType 表单组件类型，这些使用者是不能定义的，内部定义
 * @param  {object} locale 多语言
 * @param  {string} label  使用表单组件的label，如果没定义使用name值
 * @return {array}  async-validator的某个字段rules
 */
function getDefaultRules(componentType, locale, label) {
  if (!this.context.form || !this.context.useForm) {
    //没有使用form.create()，也没有使用<Form />
    return [];
  }
  var _props = this.props,
      required = _props.required,
      min = _props.min,
      max = _props.max,
      onlyLetter = _props.onlyLetter;

  var defaultRules = [];
  if (required) {
    defaultRules.push({
      required: true,
      message: (0, _sprintfJs.sprintf)(locale.afcCommon.isRequired, label)
    });
  }
  if (componentType === 'text' || componentType === 'password') {
    if (onlyLetter) {
      defaultRules.push({
        validator: function validator(rule, value, callback) {
          var errors = [];
          var pass = new RegExp('^[A-Za-z]*$').test(value);
          if (!pass) {
            errors.push({
              message: locale.afcCommon.charactersOnlyLetter
            });
          }
          callback(errors);
        }
      });
    }
    if (min || max) {
      defaultRules = defaultRules.concat(getLenthRule(min, max, locale));
    }
  }
  return defaultRules;
}